@@ -25,12 +25,26 @@
 #include "krb5.h"
 #endif
 
-OM_uint32 _gss_dummy (OM_uint32 minor_status, ...)
+OM_uint32
+_gss_dummy (OM_uint32 minor_status, ...)
 {
-  fprintf (stderr, "warning: no suitable mechanism found\n");
+  fprintf (stderr, _("warning: no suitable mechanism found\n"));
   return GSS_S_BAD_MECH;
 }
 
+OM_uint32
+_gss_dummy_display_status (OM_uint32 * minor_status,
+			   OM_uint32 status_value,
+			   int status_type,
+			   const gss_OID mech_type,
+			   OM_uint32 * message_context,
+			   gss_buffer_t status_string)
+{
+  status_string->value = strdup(_("No suitable mechanism supported"));
+  status_string->length = strlen(status_string->value);
+  return GSS_S_COMPLETE;
+}
+
 _gss_mech_api_desc _gss_mech_apis[] = {
 #ifdef USE_KERBEROS5
   {
@@ -44,7 +58,8 @@ _gss_mech_api_desc _gss_mech_apis[] = {
     gss_krb5_wrap,
     gss_krb5_unwrap,
     _gss_dummy,
-    _gss_dummy
+    _gss_dummy,
+    gss_krb5_display_status
   },
 #endif
   {
@@ -55,7 +70,8 @@ _gss_mech_api_desc _gss_mech_apis[] = {
     _gss_dummy,
     _gss_dummy,
     _gss_dummy,
-    _gss_dummy
+    _gss_dummy,
+    _gss_dummy_display_status
   }
 };
 
@@ -93,6 +109,8 @@ gss_indicate_mechs (OM_uint32 * minor_status, gss_OID_set * mech_set)
 	}
     }
 
+  if (minor_status)
+    *minor_status = 0;
   return GSS_S_COMPLETE;
 }
 
@@ -121,10 +139,12 @@ gss_inquire_names_for_mech (OM_uint32 * minor_status,
 	}
     }
 
+  if (minor_status)
+    *minor_status = 0;
   return GSS_S_COMPLETE;
 }
 
-OM_uint32
+static OM_uint32
 _gss_inquire_mechs_for_name2 (OM_uint32 * minor_status,
 			      _gss_mech_api_t mech,
 			      gss_OID name_type,
@@ -159,10 +179,12 @@ _gss_inquire_mechs_for_name2 (OM_uint32 * minor_status,
     }
   gss_release_oid_set (minor_status, &oids);
 
+  if (minor_status)
+    *minor_status = 0;
   return GSS_S_COMPLETE;
 }
 
-OM_uint32
+static OM_uint32
 _gss_inquire_mechs_for_name1 (OM_uint32 * minor_status,
 			      gss_OID name_type,
 			      gss_OID_set *mech_types)
@@ -180,6 +202,8 @@ _gss_inquire_mechs_for_name1 (OM_uint32 * minor_status,
 	return maj_stat;
     }
 
+  if (minor_status)
+    *minor_status = 0;
   return GSS_S_COMPLETE;
 }
 
@@ -203,5 +227,7 @@ gss_inquire_mechs_for_name (OM_uint32 * minor_status,
       return maj_stat;
     }
 
+  if (minor_status)
+    *minor_status = 0;
   return GSS_S_COMPLETE;
 }
