@@ -43,10 +43,15 @@ gss_import_name (OM_uint32 * minor_status,
   memcpy ((*output_name)->value, input_name_buffer->value,
 	  input_name_buffer->length);
 
-  major_stat = gss_duplicate_oid (minor_status, input_name_type,
-				  &(*output_name)->type);
-  if (major_stat != GSS_S_COMPLETE)
-    return major_stat;
+  if (input_name_type)
+    {
+      major_stat = gss_duplicate_oid (minor_status, input_name_type,
+				      &(*output_name)->type);
+      if (major_stat != GSS_S_COMPLETE)
+	return major_stat;
+    }
+  else
+    (*output_name)->type = GSS_C_NO_OID;
 
   if (minor_status)
     *minor_status = 0;
@@ -62,13 +67,17 @@ gss_display_name (OM_uint32 * minor_status,
     return GSS_S_BAD_NAME;
 
   output_name_buffer->length = input_name->length;
-  output_name_buffer->value = malloc (input_name->length);
+  output_name_buffer->value = malloc (input_name->length+1);
   if (!output_name_buffer->value)
     return GSS_S_FAILURE;
-  memcpy (output_name_buffer->value, input_name->value, input_name->length);
+  if (input_name->value)
+    memcpy (output_name_buffer->value, input_name->value, input_name->length);
 
   if (output_name_type)
-    *output_name_type = &input_name->type;
+    if (input_name->type)
+      *output_name_type = &input_name->type;
+    else
+      *output_name_type = GSS_C_NO_OID;
 
   if (minor_status)
     *minor_status = 0;
@@ -138,13 +147,16 @@ OM_uint32
 gss_duplicate_name (OM_uint32 * minor_status,
 		    const gss_name_t src_name, gss_name_t * dest_name)
 {
+  OM_uint32 maj_stat;
+
   if (src_name == GSS_C_NO_NAME)
     return GSS_S_BAD_NAME;
 
   if (!dest_name || !*dest_name)
     return GSS_S_FAILURE;
 
-  (*dest_name)->type = src_name->type; /* XXX duplicate oid? */
+  maj_stat = gss_duplicate_oid (minor_status, src_name->type,
+				&((*dest_name)->type));
   (*dest_name)->length = src_name->length;
   (*dest_name)->value = malloc(src_name->length);
   if (!(*dest_name)->value)
